#include "sfs_log.h"
#include "sfs_parameters.h"
#include "sfs_volume.h"

extern FILE *logfile;

void init_log(const char *path)
{
    // TODO error handling
    logfile = fopen(path, "a");
}

// log message to logfile
void LOG(const char *format, ...)
{
    va_list ap;
    va_start(ap, format);
    vfprintf(logfile, format, ap);
    fflush(logfile);
}

void LOG_inode(int nb)
{
    inode_t *nd = sfs_get_inode(nb);
    LOG("inode n° %d (%s):\n", nb, nd->nlinks == 0 ? "free" : "used");
    LOG("  - type: ");
    if (nd->type == S_IFREG)
        LOG("file\n");
    else if (nd->type == S_IFDIR)
        LOG("directory\n");
    else if (nd->type == S_IFLNK)
        LOG("symbolic link\n");
    else
        LOG("???\n");
    LOG("  - size: %d byte(s)\n", nd->size);
    LOG("  - pointers:\n");
    for (int i = 0; i < NB_DIRECT_BLOCKS + 1; i++) {
        LOG("      %2d => %-6d", i, nd->blocks[i]);
        if (i >= nd->size / BLOCK_SIZE + (nd->size % BLOCK_SIZE != 0))
            LOG("unused");
        LOG("\n");
    }
}
